//---------------------------------
// 2017/1/31

#include <Wire.h>
#include "INA219I2C.h"



INA219I2C::INA219I2C(uint8_t sda, uint8_t scl){
  pinSDA = sda;
  pinSCL = scl;

}


// Read 1word from register
uint16_t INA219I2C::readAddress(uint8_t addr){
  Wire.begin(pinSDA, pinSCL);
  Wire.beginTransmission(i2cAddr);
  Wire.write(addr);
  Wire.endTransmission();
  Wire.requestFrom(i2cAddr,2);
  uint8_t high = Wire.read();
  uint8_t low = Wire.read();
  return (high<<8) | low;
}


// Write 1word to register
void INA219I2C::writeAddress(uint8_t addr, uint16_t data){
  Wire.begin(pinSDA, pinSCL);
  Wire.beginTransmission(i2cAddr);
  Wire.write(addr);
  Wire.write(data>>8);
  Wire.write(data&0xFF);
  Wire.endTransmission();
}


bool INA219I2C::init(){
  writeAddress(0x0, 0x399F);
  writeAddress(0x5, cal);
  delay(1);
  if(readAddress(0x0) == 0x399F){
    return true;
  }

  return false;
}


float INA219I2C::vbus(){
  return (float)((readAddress(0x2)>>3) <<5) / 8000;  // Vbus register x 32V range / 8000
}

String INA219I2C::vbusStr(){
  char c[8];
  dtostrf(vbus(), 3, 2, c);
  return String(c);
}


float INA219I2C::current(){
  uint16_t regUint = readAddress(0x4);
  int16_t regInt;
  if(regUint<=32767){
    regInt = regUint;
  }else{
    regInt = regUint - 65536;
  }

  if(regInt > 32000 || regInt < -32000){
    return 0;
  }
  return (float)regInt * currentLSB;
}


String INA219I2C::currentStr(){
  char c[8];
  dtostrf(current(), 2, 1, c);
  return String(c);
}


uint16_t INA219I2C::power(){
  return readAddress(0x3)*powerLSB;
}

String INA219I2C::powerStr(){
  return String(power());
}









